#!/usr/bin/env python

#Written by Andrew Avila, 2011

import MySQLdb
import argparse
import os
import traceback
import sys

from pdb_subset import pdbSubset

MYSQL_HOST = '127.0.0.1'
MYSQL_USER = 'USERNAME'
MYSQL_PASSWD = 'PASSWORD'
MYSQL_DB = 'Artemis'

def DBManageConn(f):
    return lambda x, y, z : f(MySQLdb.connect(MYSQL_HOST, MYSQL_USER, MYSQL_PASSWD, MYSQL_DB), x, y, z)

def DBFetch(f):
    return lambda x, y, z : f(x.cursor(), y, z).fetchall()

@DBManageConn
def DBQueryDataFunc(conn, query, params, datafunc = lambda x, y: y):
    print query
    data = datafunc(conn, DBExecuteQuery(conn, query, params))
    conn.close()
    return data

@DBFetch
def DBExecuteQuery(cursor, query, params):
    cursor.execute(query, params)
    return cursor

def DBInsertFunc(table, nvpairs, datafunc = lambda x, y: x.insert_id()): 
    return DBQueryDataFunc("INSERT INTO %s (%s) VALUES (%s)" % (table, reduce(lambda x, y : x + ', ' + y, nvpairs.iterkeys()), MultiplyStr("%s", len(map(lambda x: x, nvpairs.itervalues())))), map(lambda x: x, nvpairs.itervalues()), datafunc)

def MultiplyStr(string, number):
    return (((string + ',')*number).strip(','))

def printx(x):
    print x

def RecursiveReceptor(dir):
    map((lambda x : RecursiveReceptor(dir + "/" + x) if os.path.isdir(dir + "/" + x) else SingleReceptor(dir + "/" + x)), os.listdir(dir))

def SingleReceptor(file):
    if len(DBQueryDataFunc("SELECT ID FROM PDB WHERE RCSB_ID = '%s'" % os.path.splitext(os.path.basename(file))[0][3:], [], lambda x, y: y)) == 0:
        try:
            f = open(file, 'r')
            UploadReceptor(os.path.splitext(os.path.basename(file))[0][3:],f.read())
            f.close()
        except:
            print "failed"
            print formatExceptionInfo()

def RecursiveReceptorInfo(dir):
    map((lambda x : RecursiveReceptorInfo(dir + "/" + x) if os.path.isdir(dir + "/" + x) else SingleReceptorInfo(dir + "/" + x)), os.listdir(dir))

def SingleReceptorInfo(file):
    if len(DBQueryDataFunc("SELECT ID FROM PDB WHERE RCSB_ID = '%s'" % os.path.splitext(os.path.basename(file))[0][3:], [], lambda x, y: y)) != 0:
        try:
            f = open(file, 'r')
            UploadReceptorInfo(os.path.splitext(os.path.basename(file))[0][3:],f.read())
            f.close()
        except:
            print "failed"
            print formatExceptionInfo()

def RecursiveLigand(dir):
    map((lambda x : RecursiveLigand(dir + "/" + x) if os.path.isdir(dir + "/" + x) else SingleLigand(dir + "/" + x)), os.listdir(dir))

def SingleLigand(file):
    try:
        f = open(file, 'r')
        UploadLigand(os.path.splitext(os.path.basename(file))[0],f.read())
        f.close()
    except:
        print "failed"
        print formatExceptionInfo()

def UploadReceptorInfo(rcsb_id, dump):
    dumpArr = dump.split('\n')
    title = reduce((lambda col, x: col + ' ' + x.replace("TITLE", "").lstrip(' ').rstrip(' ') if x[0:5] == "TITLE" else col), dumpArr, '').lstrip(' ')
    source = reduce((lambda col, x: col + ' ' + x.replace("SOURCE", "").lstrip(' ').rstrip(' ') if x[0:6] == "SOURCE" else col), dumpArr, '').lstrip(' ')
    organism = reduce((lambda col, x: col + ' ' + x[x.find(':') + 1:].lstrip(' ').rstrip(' ') if x.find("ORGANISM_SCIENTIFIC") != -1 else col), source.split(';'), '').lstrip(' ')
    DBQueryDataFunc("UPDATE PDB SET TITLE = %s, SOURCE = %s, ORGANISM = %s WHERE RCSB_ID = %s", [title, source, organism, rcsb_id], lambda x, y: y)

def UploadReceptor(rcsb_id, dump):
    pdb_id = DBInsertFunc('PDB', {'RCSB_ID': rcsb_id})
    DBInsertFunc('PDB_FILE', {'PDB_ID' : pdb_id, 'DUMP' : dump})
    UploadChains(pdb_id, dump.split('\n'))

def UploadChains(pdb_id, dump):
    chain_list = []
    for line in dump:
        atom_list=["ATOM  ","ANISOU","HETATM","TER   "]
        if line[0:6] in atom_list:
            if line[21:22] not in chain_list:
                chain_list.append(line[21:22])
                
    for chain in chain_list:
        pdb_out, chain, residues = pdbSubset(dump,chain,[0,0])
        chain_id = DBInsertFunc('CHAINS', {'PDB_ID' : pdb_id, 'NAME' : chain})
        DBInsertFunc('CHAINS_FILE', {'CHAINS_ID' : chain_id, 'DUMP' : reduce((lambda x, y: x + '\n' + y), pdb_out, '').strip('\n')})

def UploadLigand(name, dump, origin = "unknown"):
    ligands_id = DBInsertFunc('LIGANDS', {'NAME' : name, 'ORIGIN' : origin})
    DBInsertFunc('LIGANDS_FILE', {'LIGANDS_ID' : ligands_id, 'DUMP' : dump})

def formatExceptionInfo(maxTBlevel=5):
    cla, exc, trbk = sys.exc_info()
    excName = cla.__name__
    try:
        excArgs = exc.__dict__["args"]
    except KeyError:
        excArgs = "<no args>"
    excTb = traceback.format_tb(trbk, maxTBlevel)
    return (excName, excArgs, excTb)

def main():
    parser = argparse.ArgumentParser(description = "Artemis SQL Uploader")
    parser.add_argument('mode', type=str)
    parser.add_argument('file', type=str)

    args = parser.parse_args()

    if args.mode == "r":
        if os.path.isdir(args.file) == True:
            RecursiveReceptor(args.file)
        else:
            SingleReceptor(args.file)
    elif args.mode == "i":
        if os.path.isdir(args.file) == True:
            RecursiveReceptorInfo(args.file)
        else:
            SingleReceptorInfo(args.file)
    else:
        if os.path.isdir(args.file) == True:
            RecursiveLigand(args.file)
        else:
            SingleLigand(args.file)
            

if __name__ == "__main__":
    main()
